@@ -1,5 +1,5 @@
 /**
- * @file default_param.hpp
+ * @file bindings/markdown/default_param.hpp
  * @author Ryan Curtin
  *
  * Get the default value of the parameter.  This depends on
@@ -18,6 +18,9 @@
 
 #include <mlpack/bindings/cli/default_param.hpp>
 #include <mlpack/bindings/python/default_param.hpp>
+#include <mlpack/bindings/julia/default_param.hpp>
+#include <mlpack/bindings/go/default_param.hpp>
+#include <mlpack/bindings/R/default_param.hpp>
 
 namespace mlpack {
 namespace bindings {
@@ -28,7 +31,7 @@ namespace markdown {
  * printed depends on the current setting of BindingInfo::Language().
  */
 template<typename T>
-void DefaultParam(const util::ParamData& data,
+void DefaultParam(util::ParamData& data,
                   const void* /* input */,
                   void* output)
 {
@@ -42,6 +45,26 @@ void DefaultParam(const util::ParamData& data,
     *((std::string*) output) =
         python::DefaultParamImpl<typename std::remove_pointer<T>::type>(data);
   }
+  else if (BindingInfo::Language() == "julia")
+  {
+    *((std::string*) output) =
+        julia::DefaultParamImpl<typename std::remove_pointer<T>::type>(data);
+  }
+  else if (BindingInfo::Language() == "go")
+  {
+    *((std::string*) output) =
+        go::DefaultParamImpl<typename std::remove_pointer<T>::type>(data);
+  }
+  else if (BindingInfo::Language() == "r")
+  {
+    *((std::string*) output) =
+        r::DefaultParamImpl<typename std::remove_pointer<T>::type>(data);
+  }
+  else
+  {
+    throw std::invalid_argument("DefaultParam(): unknown "
+        "BindingInfo::Language() " + BindingInfo::Language() + "!");
+  }
 }
 
 } // namespace markdown
