@@ -1,5 +1,5 @@
 /**
- * @param mlpack_cli_main.hpp
+ * @param mlpack_main.hpp
  * @author Ryan Curtin
  *
  * This file, based on the value of the macro BINDING_TYPE, will define the
@@ -21,6 +21,9 @@
 #define BINDING_TYPE_CLI 0
 #define BINDING_TYPE_TEST 1
 #define BINDING_TYPE_PYX 2
+#define BINDING_TYPE_JL 3
+#define BINDING_TYPE_GO 4
+#define BINDING_TYPE_R 5
 #define BINDING_TYPE_MARKDOWN 128
 #define BINDING_TYPE_UNKNOWN -1
 
@@ -150,12 +153,6 @@ using Option = mlpack::bindings::tests::TestOption<T>;
 // testName symbol should be defined in each binding test file
 #include <mlpack/core/util/param.hpp>
 
-#undef PROGRAM_INFO
-#define PROGRAM_INFO(NAME, SHORT_DESC, DESC, ...) \
-    static mlpack::util::ProgramDoc \
-    cli_programdoc_dummy_object = mlpack::util::ProgramDoc(NAME, SHORT_DESC, \
-    []() { return DESC; }, { __VA_ARGS__ })
-
 #elif(BINDING_TYPE == BINDING_TYPE_PYX) // This is a Python binding.
 
 // Matrices are transposed on load/save.
@@ -215,11 +212,10 @@ using Option = mlpack::bindings::python::PyOption<T>;
 static const std::string testName = "";
 #include <mlpack/core/util/param.hpp>
 
-#undef PROGRAM_INFO
-#define PROGRAM_INFO(NAME, SHORT_DESC, DESC, ...) \
-    static mlpack::util::ProgramDoc \
-    cli_programdoc_dummy_object = mlpack::util::ProgramDoc(NAME, SHORT_DESC, \
-    []() { return DESC; }, { __VA_ARGS__ }); \
+#undef BINDING_NAME
+#define BINDING_NAME(NAME) static \
+    mlpack::util::ProgramName \
+    io_programname_dummy_object = mlpack::util::ProgramName(NAME); \
     namespace mlpack { \
     namespace bindings { \
     namespace python { \
@@ -237,13 +233,138 @@ PARAM_FLAG("copy_all_inputs", "If specified, all input parameters will be deep"
 
 // Nothing else needs to be defined---the binding will use mlpackMain() as-is.
 
+#elif(BINDING_TYPE == BINDING_TYPE_JL) // This is a Julia binding.
+
+// Matrices are transposed on load/save.
+#define BINDING_MATRIX_TRANSPOSED true
+
+#include <mlpack/bindings/julia/julia_option.hpp>
+#include <mlpack/bindings/julia/print_doc_functions.hpp>
+
+#define PRINT_PARAM_STRING mlpack::bindings::julia::ParamString
+#define PRINT_PARAM_VALUE mlpack::bindings::julia::PrintValue
+#define PRINT_DATASET mlpack::bindings::julia::PrintDataset
+#define PRINT_MODEL mlpack::bindings::julia::PrintModel
+#define PRINT_CALL mlpack::bindings::julia::ProgramCall
+#define BINDING_IGNORE_CHECK mlpack::bindings::julia::IgnoreCheck
+
+namespace mlpack {
+namespace util {
+
+template<typename T>
+using Option = mlpack::bindings::julia::JuliaOption<T>;
+
+}
+}
+
+static const std::string testName = "";
+#include <mlpack/core/util/param.hpp>
+
+#undef BINDING_NAME
+#define BINDING_NAME(NAME) static \
+    mlpack::util::ProgramName \
+    io_programname_dummy_object = mlpack::util::ProgramName(NAME); \
+    namespace mlpack { \
+    namespace bindings { \
+    namespace julia { \
+    std::string programName = NAME; \
+    } \
+    } \
+    }
+
+PARAM_FLAG("verbose", "Display informational messages and the full list of "
+    "parameters and timers at the end of execution.", "v");
+
+// Nothing else needs to be defined---the binding will use mlpackMain() as-is.
+
+#elif(BINDING_TYPE == BINDING_TYPE_GO) // This is a Go binding.
+
+// Matrices are transposed on load/save.
+#define BINDING_MATRIX_TRANSPOSED true
+
+#include <mlpack/bindings/go/go_option.hpp>
+#include <mlpack/bindings/go/print_doc_functions.hpp>
+
+#define PRINT_PARAM_STRING mlpack::bindings::go::ParamString
+#define PRINT_PARAM_VALUE mlpack::bindings::go::PrintValue
+#define PRINT_DATASET mlpack::bindings::go::PrintDataset
+#define PRINT_MODEL mlpack::bindings::go::PrintModel
+#define PRINT_CALL mlpack::bindings::go::ProgramCall
+#define BINDING_IGNORE_CHECK mlpack::bindings::go::IgnoreCheck
+
+namespace mlpack {
+namespace util {
+
+template<typename T>
+using Option = mlpack::bindings::go::GoOption<T>;
+
+}
+}
+
+static const std::string testName = "";
+#include <mlpack/core/util/param.hpp>
+
+#undef BINDING_NAME
+#define BINDING_NAME(NAME) static \
+    mlpack::util::ProgramName \
+    io_programname_dummy_object = mlpack::util::ProgramName(NAME); \
+    namespace mlpack { \
+    namespace bindings { \
+    namespace go { \
+    std::string programName = NAME; \
+    } \
+    } \
+    }
+
+PARAM_FLAG("verbose", "Display informational messages and the full list of "
+    "parameters and timers at the end of execution.", "v");
+
+// Nothing else needs to be defined---the binding will use mlpackMain() as-is.
+
+#elif(BINDING_TYPE == BINDING_TYPE_R) // This is a R binding.
+
+// This doesn't actually matter for this binding type.
+#define BINDING_MATRIX_TRANSPOSED true
+
+#include <mlpack/bindings/R/R_option.hpp>
+#include <mlpack/bindings/R/print_doc_functions.hpp>
+
+#define PRINT_PARAM_STRING mlpack::bindings::r::ParamString
+#define PRINT_PARAM_VALUE mlpack::bindings::r::PrintValue
+#define PRINT_DATASET mlpack::bindings::r::PrintDataset
+#define PRINT_MODEL mlpack::bindings::r::PrintModel
+#define PRINT_CALL(...) mlpack::bindings::r::ProgramCall(false, __VA_ARGS__)
+#define BINDING_IGNORE_CHECK mlpack::bindings::r::IgnoreCheck
+
+namespace mlpack {
+namespace util {
+
+template<typename T>
+using Option = mlpack::bindings::r::ROption<T>;
+
+}
+}
+
+static const std::string testName = "";
+#include <mlpack/core/util/param.hpp>
+
+PARAM_FLAG("verbose", "Display informational messages and the full list of "
+    "parameters and timers at the end of execution.", "v");
+
+// Nothing else needs to be defined---the binding will use mlpackMain() as-is.
+
 #elif BINDING_TYPE == BINDING_TYPE_MARKDOWN
 
-// We use BINDING_NAME in PROGRAM_INFO() so it needs to be defined.
-#ifndef BINDING_NAME
-  #error "BINDING_NAME must be defined when BINDING_TYPE is Markdown!"
+// We use MARKDOWN_BINDING_NAME in BINDING_NAME(), BINDING_SHORT_DESC(),
+// BINDING_LONG_DESC(), BINDING_EXAMPLE() and BINDING_SEE_ALSO()
+// so it needs to be defined.
+#ifndef MARKDOWN_BINDING_NAME
+  #error "MARKDOWN_BINDING_NAME must be defined when BINDING_TYPE is Markdown!"
 #endif
 
+// This value doesn't actually matter, but it needs to be defined as something.
+#define BINDING_MATRIX_TRANSPOSED true
+
 #include <mlpack/bindings/markdown/md_option.hpp>
 #include <mlpack/bindings/markdown/print_doc_functions.hpp>
 
@@ -301,12 +422,55 @@ using Option = mlpack::bindings::markdown::MDOption<T>;
 #include <mlpack/core/util/param.hpp>
 #include <mlpack/bindings/markdown/program_doc_wrapper.hpp>
 
-#undef PROGRAM_INFO
-#define PROGRAM_INFO(NAME, SHORT_DESC, DESC, ...) static \
-    mlpack::bindings::markdown::ProgramDocWrapper \
-    cli_programdoc_dummy_object = \
-    mlpack::bindings::markdown::ProgramDocWrapper(BINDING_NAME, NAME, \
-    SHORT_DESC, []() { return DESC; }, { __VA_ARGS__ }); \
+#undef BINDING_NAME
+#undef BINDING_SHORT_DESC
+#undef BINDING_LONG_DESC
+#undef BINDING_EXAMPLE
+#undef BINDING_SEE_ALSO
+
+#define BINDING_NAME(NAME) static \
+    mlpack::bindings::markdown::ProgramNameWrapper \
+    io_programname_dummy_object = \
+    mlpack::bindings::markdown::ProgramNameWrapper( \
+    MARKDOWN_BINDING_NAME, NAME);
+
+#define BINDING_SHORT_DESC(SHORT_DESC) static \
+    mlpack::bindings::markdown::ShortDescriptionWrapper \
+    io_programshort_desc_dummy_object = \
+    mlpack::bindings::markdown::ShortDescriptionWrapper( \
+    MARKDOWN_BINDING_NAME, SHORT_DESC);
+
+#define BINDING_LONG_DESC(LONG_DESC) static \
+    mlpack::bindings::markdown::LongDescriptionWrapper \
+    io_programlong_desc_dummy_object = \
+    mlpack::bindings::markdown::LongDescriptionWrapper( \
+    MARKDOWN_BINDING_NAME, []() { return std::string(LONG_DESC); });
+
+#ifdef __COUNTER__
+  #define BINDING_EXAMPLE(EXAMPLE) static \
+      mlpack::bindings::markdown::ExampleWrapper \
+      JOIN(io_programexample_dummy_object_, __COUNTER__) = \
+      mlpack::bindings::markdown::ExampleWrapper(MARKDOWN_BINDING_NAME, \
+      []() { return(std::string(EXAMPLE)); });
+
+  #define BINDING_SEE_ALSO(DESCRIPTION, LINK) static \
+      mlpack::bindings::markdown::SeeAlsoWrapper \
+      JOIN(io_programsee_also_dummy_object_, __COUNTER__) = \
+      mlpack::bindings::markdown::SeeAlsoWrapper(MARKDOWN_BINDING_NAME, \
+      DESCRIPTION, LINK);
+#else
+  #define BINDING_EXAMPLE(EXAMPLE) static \
+      mlpack::bindings::markdown::ExampleWrapper \
+      JOIN(JOIN(io_programexample_dummy_object_, __LINE__), opt) = \
+      mlpack::bindings::markdown::ExampleWrapper(MARKDOWN_BINDING_NAME, \
+      []() { return(std::string(EXAMPLE)); });
+
+  #define BINDING_SEE_ALSO(DESCRIPTION, LINK) static \
+      mlpack::bindings::markdown::SeeAlsoWrapper \
+      JOIN(JOIN(io_programsee_also_dummy_object_, __LINE__), opt) = \
+      mlpack::bindings::markdown::SeeAlsoWrapper(MARKDOWN_BINDING_NAME, \
+      DESCRIPTION, LINK);
+#endif
 
 PARAM_FLAG("verbose", "Display informational messages and the full list of "
     "parameters and timers at the end of execution.", "v");
