@@ -1,5 +1,5 @@
 /**
- * @file convolution_impl.hpp
+ * @file methods/ann/layer/convolution_impl.hpp
  * @author Marcus Edel
  *
  * Implementation of the Convolution module class.
@@ -52,29 +52,90 @@ Convolution<
 >::Convolution(
     const size_t inSize,
     const size_t outSize,
-    const size_t kW,
-    const size_t kH,
-    const size_t dW,
-    const size_t dH,
+    const size_t kernelWidth,
+    const size_t kernelHeight,
+    const size_t strideWidth,
+    const size_t strideHeight,
     const size_t padW,
     const size_t padH,
     const size_t inputWidth,
-    const size_t inputHeight) :
+    const size_t inputHeight,
+    const std::string& paddingType) :
+    Convolution(
+      inSize,
+      outSize,
+      kernelWidth,
+      kernelHeight,
+      strideWidth,
+      strideHeight,
+      std::tuple<size_t, size_t>(padW, padW),
+      std::tuple<size_t, size_t>(padH, padH),
+      inputWidth,
+      inputHeight,
+      paddingType)
+{
+  // Nothing to do here.
+}
+
+template<
+    typename ForwardConvolutionRule,
+    typename BackwardConvolutionRule,
+    typename GradientConvolutionRule,
+    typename InputDataType,
+    typename OutputDataType
+>
+Convolution<
+    ForwardConvolutionRule,
+    BackwardConvolutionRule,
+    GradientConvolutionRule,
+    InputDataType,
+    OutputDataType
+>::Convolution(
+    const size_t inSize,
+    const size_t outSize,
+    const size_t kernelWidth,
+    const size_t kernelHeight,
+    const size_t strideWidth,
+    const size_t strideHeight,
+    const std::tuple<size_t, size_t>& padW,
+    const std::tuple<size_t, size_t>& padH,
+    const size_t inputWidth,
+    const size_t inputHeight,
+    const std::string& paddingType) :
     inSize(inSize),
     outSize(outSize),
-    kW(kW),
-    kH(kH),
-    dW(dW),
-    dH(dH),
-    padW(padW),
-    padH(padH),
+    kernelWidth(kernelWidth),
+    kernelHeight(kernelHeight),
+    strideWidth(strideWidth),
+    strideHeight(strideHeight),
+    padWLeft(std::get<0>(padW)),
+    padWRight(std::get<1>(padW)),
+    padHBottom(std::get<1>(padH)),
+    padHTop(std::get<0>(padH)),
     inputWidth(inputWidth),
     inputHeight(inputHeight),
     outputWidth(0),
     outputHeight(0)
 {
-  weights.set_size((outSize * inSize * kW * kH) + outSize, 1);
-  padding = new Padding<>(padW, padW, padH, padH);
+  weights.set_size(WeightSize(), 1);
+
+  // Transform paddingType to lowercase.
+  std::string paddingTypeLow = paddingType;
+  util::ToLower(paddingType, paddingTypeLow);
+
+  if (paddingTypeLow == "valid")
+  {
+    padWLeft = 0;
+    padWRight = 0;
+    padHTop = 0;
+    padHBottom = 0;
+  }
+  else if (paddingTypeLow == "same")
+  {
+    InitializeSamePadding();
+  }
+
+  padding = ann::Padding<>(padWLeft, padWRight, padHTop, padHBottom);
 }
 
 template<
@@ -92,7 +153,7 @@ void Convolution<
     OutputDataType
 >::Reset()
 {
-    weight = arma::cube(weights.memptr(), kW, kH,
+    weight = arma::cube(weights.memptr(), kernelWidth, kernelHeight,
         outSize * inSize, false, false);
     bias = arma::mat(weights.memptr() + weight.n_elem,
         outSize, 1, false, false);
@@ -112,26 +173,27 @@ void Convolution<
     GradientConvolutionRule,
     InputDataType,
     OutputDataType
->::Forward(const arma::Mat<eT>&& input, arma::Mat<eT>&& output)
+>::Forward(const arma::Mat<eT>& input, arma::Mat<eT>& output)
 {
   batchSize = input.n_cols;
-  inputTemp = arma::cube(const_cast<arma::Mat<eT>&&>(input).memptr(),
+  arma::cube inputTemp(const_cast<arma::Mat<eT>&>(input).memptr(),
       inputWidth, inputHeight, inSize * batchSize, false, false);
 
-  if (padW != 0 || padH != 0)
+  if (padWLeft != 0 || padWRight != 0 || padHTop != 0 || padHBottom != 0)
   {
-    inputPaddedTemp.set_size(inputTemp.n_rows + padW * 2,
-        inputTemp.n_cols + padH * 2, inputTemp.n_slices);
+    inputPaddedTemp.set_size(inputTemp.n_rows + padWLeft + padWRight,
+        inputTemp.n_cols + padHTop + padHBottom, inputTemp.n_slices);
 
     for (size_t i = 0; i < inputTemp.n_slices; ++i)
     {
-      padding->Forward(std::move(inputTemp.slice(i)),
-          std::move(inputPaddedTemp.slice(i)));
+      padding.Forward(inputTemp.slice(i), inputPaddedTemp.slice(i));
     }
   }
 
-  size_t wConv = ConvOutSize(inputWidth, kW, dW, padW);
-  size_t hConv = ConvOutSize(inputHeight, kH, dH, padH);
+  size_t wConv = ConvOutSize(inputWidth, kernelWidth, strideWidth, padWLeft,
+      padWRight);
+  size_t hConv = ConvOutSize(inputHeight, kernelHeight, strideHeight, padHTop,
+      padHBottom);
 
   output.set_size(wConv * hConv * outSize, batchSize);
   outputTemp = arma::Cube<eT>(output.memptr(), wConv, hConv,
@@ -151,15 +213,17 @@ void Convolution<
     {
       arma::Mat<eT> convOutput;
 
-      if (padW != 0 || padH != 0)
+      if (padWLeft != 0 || padWRight != 0 || padHTop != 0 || padHBottom != 0)
       {
         ForwardConvolutionRule::Convolution(inputPaddedTemp.slice(inMap +
-            batchCount * inSize), weight.slice(outMapIdx), convOutput, dW, dH);
+            batchCount * inSize), weight.slice(outMapIdx), convOutput,
+            strideWidth, strideHeight);
       }
       else
       {
         ForwardConvolutionRule::Convolution(inputTemp.slice(inMap +
-            batchCount * inSize), weight.slice(outMapIdx), convOutput, dW, dH);
+            batchCount * inSize), weight.slice(outMapIdx), convOutput,
+            strideWidth, strideHeight);
       }
 
       outputTemp.slice(outMap) += convOutput;
@@ -187,14 +251,14 @@ void Convolution<
     InputDataType,
     OutputDataType
 >::Backward(
-    const arma::Mat<eT>&& /* input */, arma::Mat<eT>&& gy, arma::Mat<eT>&& g)
+    const arma::Mat<eT>& /* input */, const arma::Mat<eT>& gy, arma::Mat<eT>& g)
 {
-  arma::cube mappedError(gy.memptr(), outputWidth, outputHeight,
-      outSize * batchSize, false, false);
+  arma::cube mappedError(((arma::Mat<eT>&) gy).memptr(), outputWidth,
+      outputHeight, outSize * batchSize, false, false);
 
-  g.set_size(inputTemp.n_rows * inputTemp.n_cols * inSize, batchSize);
-  gTemp = arma::Cube<eT>(g.memptr(), inputTemp.n_rows,
-      inputTemp.n_cols, inputTemp.n_slices, false, false);
+  g.set_size(inputWidth * inputHeight * inSize, batchSize);
+  gTemp = arma::Cube<eT>(g.memptr(), inputWidth, inputHeight,
+      inSize * batchSize, false, false);
   gTemp.zeros();
 
   for (size_t outMap = 0, outMapIdx = 0, batchCount = 0; outMap <
@@ -212,13 +276,12 @@ void Convolution<
       Rotate180(weight.slice(outMapIdx), rotatedFilter);
 
       BackwardConvolutionRule::Convolution(mappedError.slice(outMap),
-          rotatedFilter, output, dW, dH);
+          rotatedFilter, output, strideWidth, strideHeight);
 
-      if (padW != 0 || padH != 0)
+      if (padWLeft != 0 || padWRight != 0 || padHTop != 0 || padHBottom != 0)
       {
-        gTemp.slice(inMap + batchCount * inSize) += output.submat(padW, padH,
-            padW + gTemp.n_rows - 1,
-            padH + gTemp.n_cols - 1);
+        gTemp.slice(inMap + batchCount * inSize) += output.submat(padWLeft,
+            padHTop, padWLeft + gTemp.n_rows - 1, padHTop + gTemp.n_cols - 1);
       }
       else
       {
@@ -243,12 +306,14 @@ void Convolution<
     InputDataType,
     OutputDataType
 >::Gradient(
-    const arma::Mat<eT>&& /* input */,
-    arma::Mat<eT>&& error,
-    arma::Mat<eT>&& gradient)
+    const arma::Mat<eT>& input,
+    const arma::Mat<eT>& error,
+    arma::Mat<eT>& gradient)
 {
-  arma::cube mappedError(error.memptr(), outputWidth,
+  arma::cube mappedError(((arma::Mat<eT>&) error).memptr(), outputWidth,
       outputHeight, outSize * batchSize, false, false);
+  arma::cube inputTemp(((arma::Mat<eT>&) input).memptr(), inputWidth,
+      inputHeight, inSize * batchSize, false, false);
 
   gradient.set_size(weights.n_elem, 1);
   gradientTemp = arma::Cube<eT>(gradient.memptr(), weight.n_rows,
@@ -267,7 +332,7 @@ void Convolution<
     for (size_t inMap = 0; inMap < inSize; inMap++, outMapIdx++)
     {
       arma::Mat<eT> inputSlice;
-      if (padW != 0 || padH != 0)
+      if (padWLeft != 0 || padWRight != 0 || padHTop != 0 || padHBottom != 0)
       {
         inputSlice = inputPaddedTemp.slice(inMap + batchCount * inSize);
       }
@@ -280,7 +345,7 @@ void Convolution<
 
       arma::Mat<eT> output;
       GradientConvolutionRule::Convolution(inputSlice, deltaSlice,
-          output, dW, dH);
+          output, strideWidth, strideHeight);
 
       if (gradientTemp.n_rows < output.n_rows ||
           gradientTemp.n_cols < output.n_cols)
@@ -319,25 +384,61 @@ void Convolution<
     GradientConvolutionRule,
     InputDataType,
     OutputDataType
->::serialize(
-    Archive& ar, const unsigned int /* version */)
+>::serialize(Archive& ar, const unsigned int version)
 {
   ar & BOOST_SERIALIZATION_NVP(inSize);
   ar & BOOST_SERIALIZATION_NVP(outSize);
   ar & BOOST_SERIALIZATION_NVP(batchSize);
-  ar & BOOST_SERIALIZATION_NVP(kW);
-  ar & BOOST_SERIALIZATION_NVP(kH);
-  ar & BOOST_SERIALIZATION_NVP(dW);
-  ar & BOOST_SERIALIZATION_NVP(dH);
-  ar & BOOST_SERIALIZATION_NVP(padW);
-  ar & BOOST_SERIALIZATION_NVP(padH);
+  ar & BOOST_SERIALIZATION_NVP(kernelWidth);
+  ar & BOOST_SERIALIZATION_NVP(kernelHeight);
+  ar & BOOST_SERIALIZATION_NVP(strideWidth);
+  ar & BOOST_SERIALIZATION_NVP(strideHeight);
+  ar & BOOST_SERIALIZATION_NVP(padWLeft);
+  ar & BOOST_SERIALIZATION_NVP(padWRight);
+  ar & BOOST_SERIALIZATION_NVP(padHBottom);
+  ar & BOOST_SERIALIZATION_NVP(padHTop);
   ar & BOOST_SERIALIZATION_NVP(inputWidth);
   ar & BOOST_SERIALIZATION_NVP(inputHeight);
   ar & BOOST_SERIALIZATION_NVP(outputWidth);
   ar & BOOST_SERIALIZATION_NVP(outputHeight);
 
+  if (version > 0)
+    ar & BOOST_SERIALIZATION_NVP(padding);
+
   if (Archive::is_loading::value)
-    weights.set_size((outSize * inSize * kW * kH) + outSize, 1);
+  {
+    weights.set_size((outSize * inSize * kernelWidth * kernelHeight) + outSize,
+        1);
+  }
+}
+
+template<
+    typename ForwardConvolutionRule,
+    typename BackwardConvolutionRule,
+    typename GradientConvolutionRule,
+    typename InputDataType,
+    typename OutputDataType
+>
+void Convolution<
+    ForwardConvolutionRule,
+    BackwardConvolutionRule,
+    GradientConvolutionRule,
+    InputDataType,
+    OutputDataType
+>::InitializeSamePadding()
+{
+  /*
+   * Using O = (W - F + 2P) / s + 1;
+   */
+  size_t totalVerticalPadding = (strideWidth - 1) * inputWidth + kernelWidth -
+      strideWidth;
+  size_t totalHorizontalPadding = (strideHeight - 1) * inputHeight +
+      kernelHeight - strideHeight;
+
+  padWLeft = totalVerticalPadding / 2;
+  padWRight = totalVerticalPadding - totalVerticalPadding / 2;
+  padHTop = totalHorizontalPadding / 2;
+  padHBottom = totalHorizontalPadding - totalHorizontalPadding / 2;
 }
 
 } // namespace ann
