@@ -1,5 +1,5 @@
 /**
- * @file layer_traits.hpp
+ * @file methods/ann/layer/layer_traits.hpp
  * @author Marcus Edel
  *
  * This provides the LayerTraits class, a template class to get information
@@ -116,6 +116,10 @@ HAS_MEM_FUNC(Run, HasRunCheck);
 // can use with SFINAE to catch when a type has a Bias() function.
 HAS_MEM_FUNC(Bias, HasBiasCheck);
 
+// This gives us a HasMaxIterationsC<T, U> type (where U is a function pointer)
+// we can use with SFINAE to catch when a type has a MaxIterations() function.
+HAS_MEM_FUNC(MaxIterations, HasMaxIterations);
+
 } // namespace ann
 } // namespace mlpack
 
