@@ -1,5 +1,5 @@
 /**
- * @file decision_tree_impl.hpp
+ * @file methods/decision_tree/decision_tree_impl.hpp
  * @author Ryan Curtin
  *
  * Implementation of generic decision tree class.
@@ -12,6 +12,8 @@
 #ifndef MLPACK_METHODS_DECISION_TREE_DECISION_TREE_IMPL_HPP
 #define MLPACK_METHODS_DECISION_TREE_DECISION_TREE_IMPL_HPP
 
+#include "decision_tree.hpp"
+
 namespace mlpack {
 namespace tree {
 
@@ -35,6 +37,7 @@ DecisionTree<FitnessFunction,
     const size_t numClasses,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector)
 {
   using TrueMatType = typename std::decay<MatType>::type;
@@ -50,7 +53,8 @@ DecisionTree<FitnessFunction,
   // Pass off work to the Train() method.
   arma::rowvec weights; // Fake weights, not used.
   Train<false>(tmpData, 0, tmpData.n_cols, datasetInfo, tmpLabels, numClasses,
-      weights, minimumLeafSize, minimumGainSplit, dimensionSelector);
+      weights, minimumLeafSize, minimumGainSplit, maximumDepth,
+      dimensionSelector);
 }
 
 //! Construct and train.
@@ -72,6 +76,7 @@ DecisionTree<FitnessFunction,
     const size_t numClasses,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector)
 {
   using TrueMatType = typename std::decay<MatType>::type;
@@ -87,7 +92,7 @@ DecisionTree<FitnessFunction,
   // Pass off work to the Train() method.
   arma::rowvec weights; // Fake weights, not used.
   Train<false>(tmpData, 0, tmpData.n_cols, tmpLabels, numClasses, weights,
-      minimumLeafSize, minimumGainSplit, dimensionSelector);
+      minimumLeafSize, minimumGainSplit, maximumDepth, dimensionSelector);
 }
 
 //! Construct and train with weights.
@@ -111,11 +116,10 @@ DecisionTree<FitnessFunction,
     WeightsType weights,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector,
-    const std::enable_if_t<
-        arma::is_arma_type<
-        typename std::remove_reference<
-        WeightsType>::type>::value>*)
+    const std::enable_if_t<arma::is_arma_type<
+        typename std::remove_reference<WeightsType>::type>::value>*)
 {
   using TrueMatType = typename std::decay<MatType>::type;
   using TrueLabelsType = typename std::decay<LabelsType>::type;
@@ -131,7 +135,49 @@ DecisionTree<FitnessFunction,
 
   // Pass off work to the weighted Train() method.
   Train<true>(tmpData, 0, tmpData.n_cols, datasetInfo, tmpLabels, numClasses,
-      tmpWeights, minimumLeafSize, minimumGainSplit, dimensionSelector);
+      tmpWeights, minimumLeafSize, minimumGainSplit, maximumDepth,
+      dimensionSelector);
+}
+
+//! Construct and train with weights.
+template<typename FitnessFunction,
+        template<typename> class NumericSplitType,
+        template<typename> class CategoricalSplitType,
+        typename DimensionSelectionType,
+        typename ElemType,
+        bool NoRecursion>
+template<typename MatType, typename LabelsType, typename WeightsType>
+DecisionTree<FitnessFunction,
+        NumericSplitType,
+        CategoricalSplitType,
+        DimensionSelectionType,
+        ElemType,
+        NoRecursion>::DecisionTree(
+    const DecisionTree& other,
+    MatType data,
+    const data::DatasetInfo& datasetInfo,
+    LabelsType labels,
+    const size_t numClasses,
+    WeightsType weights,
+    const size_t minimumLeafSize,
+    const double minimumGainSplit,
+    const std::enable_if_t<arma::is_arma_type<
+        typename std::remove_reference<WeightsType>::type>::value>*):
+        NumericAuxiliarySplitInfo(other),
+        CategoricalAuxiliarySplitInfo(other)
+{
+  using TrueMatType = typename std::decay<MatType>::type;
+  using TrueLabelsType = typename std::decay<LabelsType>::type;
+  using TrueWeightsType = typename std::decay<WeightsType>::type;
+
+  // Copy or move data.
+  TrueMatType tmpData(std::move(data));
+  TrueLabelsType tmpLabels(std::move(labels));
+  TrueWeightsType tmpWeights(std::move(weights));
+
+  // Pass off work to the weighted Train() method.
+  Train<true>(tmpData, 0, tmpData.n_cols, datasetInfo, tmpLabels, numClasses,
+              tmpWeights, minimumLeafSize, minimumGainSplit);
 }
 
 //! Construct and train with weights.
@@ -154,6 +200,7 @@ DecisionTree<FitnessFunction,
     WeightsType weights,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector,
     const std::enable_if_t<
         arma::is_arma_type<
@@ -174,7 +221,53 @@ DecisionTree<FitnessFunction,
 
   // Pass off work to the weighted Train() method.
   Train<true>(tmpData, 0, tmpData.n_cols, tmpLabels, numClasses, tmpWeights,
-      minimumLeafSize, minimumGainSplit, dimensionSelector);
+      minimumLeafSize, minimumGainSplit, maximumDepth, dimensionSelector);
+}
+
+//! Construct and train with weights.
+template<typename FitnessFunction,
+        template<typename> class NumericSplitType,
+        template<typename> class CategoricalSplitType,
+        typename DimensionSelectionType,
+        typename ElemType,
+        bool NoRecursion>
+template<typename MatType, typename LabelsType, typename WeightsType>
+DecisionTree<FitnessFunction,
+        NumericSplitType,
+        CategoricalSplitType,
+        DimensionSelectionType,
+        ElemType,
+        NoRecursion>::DecisionTree(
+    const DecisionTree& other,
+    MatType data,
+    LabelsType labels,
+    const size_t numClasses,
+    WeightsType weights,
+    const size_t minimumLeafSize,
+    const double minimumGainSplit,
+    const size_t maximumDepth,
+    DimensionSelectionType dimensionSelector,
+    const std::enable_if_t<arma::is_arma_type<
+        typename std::remove_reference<
+        WeightsType>::type>::value>*):
+        NumericAuxiliarySplitInfo(other),
+        CategoricalAuxiliarySplitInfo(other)  // other info does need to copy
+{
+  using TrueMatType = typename std::decay<MatType>::type;
+  using TrueLabelsType = typename std::decay<LabelsType>::type;
+  using TrueWeightsType = typename std::decay<WeightsType>::type;
+
+  // Copy or move data.
+  TrueMatType tmpData(std::move(data));
+  TrueLabelsType tmpLabels(std::move(labels));
+  TrueWeightsType tmpWeights(std::move(weights));
+
+  // Set the correct dimensionality for the dimension selector.
+  dimensionSelector.Dimensions() = tmpData.n_rows;
+
+  // Pass off work to the weighted Train() method.
+  Train<true>(tmpData, 0, tmpData.n_cols, tmpLabels, numClasses, tmpWeights,
+      minimumLeafSize, minimumGainSplit, maximumDepth, dimensionSelector);
 }
 
 //! Construct, don't train.
@@ -372,6 +465,7 @@ double DecisionTree<FitnessFunction,
     const size_t numClasses,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector)
 {
   // Sanity check on data.
@@ -397,7 +491,7 @@ double DecisionTree<FitnessFunction,
   // Pass off work to the Train() method.
   arma::rowvec weights; // Fake weights, not used.
   return Train<false>(tmpData, 0, tmpData.n_cols, datasetInfo, tmpLabels,
-      numClasses, weights, minimumLeafSize, minimumGainSplit,
+      numClasses, weights, minimumLeafSize, minimumGainSplit, maximumDepth,
       dimensionSelector);
 }
 
@@ -420,6 +514,7 @@ double DecisionTree<FitnessFunction,
     const size_t numClasses,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector)
 {
   // Sanity check on data.
@@ -445,7 +540,8 @@ double DecisionTree<FitnessFunction,
   // Pass off work to the Train() method.
   arma::rowvec weights; // Fake weights, not used.
   return Train<false>(tmpData, 0, tmpData.n_cols, tmpLabels, numClasses,
-      weights, minimumLeafSize, minimumGainSplit, dimensionSelector);
+      weights, minimumLeafSize, minimumGainSplit, maximumDepth,
+      dimensionSelector);
 }
 
 //! Train on the given weighted data.
@@ -469,6 +565,7 @@ double DecisionTree<FitnessFunction,
     WeightsType weights,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector,
     const std::enable_if_t<
         arma::is_arma_type<
@@ -499,7 +596,7 @@ double DecisionTree<FitnessFunction,
 
   // Pass off work to the Train() method.
   return Train<true>(tmpData, 0, tmpData.n_cols, datasetInfo, tmpLabels,
-      numClasses, tmpWeights, minimumLeafSize, minimumGainSplit,
+      numClasses, tmpWeights, minimumLeafSize, minimumGainSplit, maximumDepth,
       dimensionSelector);
 }
 
@@ -523,6 +620,7 @@ double DecisionTree<FitnessFunction,
     WeightsType weights,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType dimensionSelector,
     const std::enable_if_t<
         arma::is_arma_type<
@@ -553,10 +651,11 @@ double DecisionTree<FitnessFunction,
 
   // Pass off work to the Train() method.
   return Train<true>(tmpData, 0, tmpData.n_cols, tmpLabels, numClasses,
-      tmpWeights, minimumLeafSize, minimumGainSplit, dimensionSelector);
+      tmpWeights, minimumLeafSize, minimumGainSplit, maximumDepth,
+      dimensionSelector);
 }
 
-//! Train on the given data.
+//! Train on the given data, assuming all dimensions are numeric.
 template<typename FitnessFunction,
          template<typename> class NumericSplitType,
          template<typename> class CategoricalSplitType,
@@ -579,6 +678,7 @@ double DecisionTree<FitnessFunction,
     arma::rowvec& weights,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType& dimensionSelector)
 {
   // Clear children if needed.
@@ -597,48 +697,52 @@ double DecisionTree<FitnessFunction,
       UseWeights ? weights.subvec(begin, begin + count - 1) : weights);
   size_t bestDim = datasetInfo.Dimensionality(); // This means "no split".
   const size_t end = dimensionSelector.End();
-  for (size_t i = dimensionSelector.Begin(); i != end;
-       i = dimensionSelector.Next())
+
+  if (maximumDepth != 1)
   {
-    double dimGain = -DBL_MAX;
-    if (datasetInfo.Type(i) == data::Datatype::categorical)
+    for (size_t i = dimensionSelector.Begin(); i != end;
+         i = dimensionSelector.Next())
     {
-      dimGain = CategoricalSplit::template SplitIfBetter<UseWeights>(bestGain,
-          data.cols(begin, begin + count - 1).row(i),
-          datasetInfo.NumMappings(i),
-          labels.subvec(begin, begin + count - 1),
-          numClasses,
-          UseWeights ? weights.subvec(begin, begin + count - 1) : weights,
-          minimumLeafSize,
-          minimumGainSplit,
-          classProbabilities,
-          *this);
+      double dimGain = -DBL_MAX;
+      if (datasetInfo.Type(i) == data::Datatype::categorical)
+      {
+        dimGain = CategoricalSplit::template SplitIfBetter<UseWeights>(bestGain,
+            data.cols(begin, begin + count - 1).row(i),
+            datasetInfo.NumMappings(i),
+            labels.subvec(begin, begin + count - 1),
+            numClasses,
+            UseWeights ? weights.subvec(begin, begin + count - 1) : weights,
+            minimumLeafSize,
+            minimumGainSplit,
+            classProbabilities,
+            *this);
+      }
+      else if (datasetInfo.Type(i) == data::Datatype::numeric)
+      {
+        dimGain = NumericSplit::template SplitIfBetter<UseWeights>(bestGain,
+            data.cols(begin, begin + count - 1).row(i),
+            labels.subvec(begin, begin + count - 1),
+            numClasses,
+            UseWeights ? weights.subvec(begin, begin + count - 1) : weights,
+            minimumLeafSize,
+            minimumGainSplit,
+            classProbabilities,
+            *this);
+      }
+
+      // If the splitter reported that it did not split, move to the next
+      // dimension.
+      if (dimGain == DBL_MAX)
+        continue;
+
+      // Was there an improvement?  If so mark that it's the new best dimension.
+      bestDim = i;
+      bestGain = dimGain;
+
+      // If the gain is the best possible, no need to keep looking.
+      if (bestGain >= 0.0)
+        break;
     }
-    else if (datasetInfo.Type(i) == data::Datatype::numeric)
-    {
-      dimGain = NumericSplit::template SplitIfBetter<UseWeights>(bestGain,
-          data.cols(begin, begin + count - 1).row(i),
-          labels.subvec(begin, begin + count - 1),
-          numClasses,
-          UseWeights ? weights.subvec(begin, begin + count - 1) : weights,
-          minimumLeafSize,
-          minimumGainSplit,
-          classProbabilities,
-          *this);
-    }
-
-    // If the splitter reported that it did not split, move to the next
-    // dimension.
-    if (dimGain == DBL_MAX)
-      continue;
-
-    // Was there an improvement?  If so mark that it's the new best dimension.
-    bestDim = i;
-    bestGain = dimGain;
-
-    // If the gain is the best possible, no need to keep looking.
-    if (bestGain >= 0.0)
-      break;
   }
 
   // Did we split or not?  If so, then split the data and create the children.
@@ -707,14 +811,15 @@ double DecisionTree<FitnessFunction,
         child->Train<UseWeights>(data, currentChildBegin,
             currentCol - currentChildBegin, datasetInfo, labels, numClasses,
             weights, currentCol - currentChildBegin, minimumGainSplit,
-            dimensionSelector);
+            maximumDepth - 1, dimensionSelector);
       }
       else
       {
         // During recursion entropy of child node may change.
         double childGain = child->Train<UseWeights>(data, currentChildBegin,
             currentCol - currentChildBegin, datasetInfo, labels, numClasses,
-            weights, minimumLeafSize, minimumGainSplit, dimensionSelector);
+            weights, minimumLeafSize, minimumGainSplit, maximumDepth - 1,
+            dimensionSelector);
         bestGain += double(childCounts[i]) / double(count) * (-childGain);
       }
       children.push_back(child);
@@ -732,6 +837,7 @@ double DecisionTree<FitnessFunction,
         numClasses,
         UseWeights ? weights.subvec(begin, begin + count - 1) : weights);
   }
+
   return -bestGain;
 }
 
@@ -757,6 +863,7 @@ double DecisionTree<FitnessFunction,
     arma::rowvec& weights,
     const size_t minimumLeafSize,
     const double minimumGainSplit,
+    const size_t maximumDepth,
     DimensionSelectionType& dimensionSelector)
 {
   // Clear children if needed.
@@ -777,33 +884,37 @@ double DecisionTree<FitnessFunction,
       numClasses,
       UseWeights ? weights.subvec(begin, begin + count - 1) : weights);
   size_t bestDim = data.n_rows; // This means "no split".
-  for (size_t i = dimensionSelector.Begin(); i != dimensionSelector.End();
-       i = dimensionSelector.Next())
+
+  if (maximumDepth != 1)
   {
-    const double dimGain = NumericSplitType<FitnessFunction>::template
-        SplitIfBetter<UseWeights>(bestGain,
-                                  data.cols(begin, begin + count - 1).row(i),
-                                  labels.cols(begin, begin + count - 1),
-                                  numClasses,
-                                  UseWeights ?
-                                      weights.cols(begin, begin + count - 1) :
-                                      weights,
-                                  minimumLeafSize,
-                                  minimumGainSplit,
-                                  classProbabilities,
-                                  *this);
+    for (size_t i = dimensionSelector.Begin(); i != dimensionSelector.End();
+         i = dimensionSelector.Next())
+    {
+      const double dimGain = NumericSplitType<FitnessFunction>::template
+          SplitIfBetter<UseWeights>(bestGain,
+                                    data.cols(begin, begin + count - 1).row(i),
+                                    labels.cols(begin, begin + count - 1),
+                                    numClasses,
+                                    UseWeights ?
+                                        weights.cols(begin, begin + count - 1) :
+                                        weights,
+                                    minimumLeafSize,
+                                    minimumGainSplit,
+                                    classProbabilities,
+                                    *this);
 
-    // If the splitter did not report that it improved, then move to the next
-    // dimension.
-    if (dimGain == DBL_MAX)
-      continue;
+      // If the splitter did not report that it improved, then move to the next
+      // dimension.
+      if (dimGain == DBL_MAX)
+        continue;
 
-    bestDim = i;
-    bestGain = dimGain;
+      bestDim = i;
+      bestGain = dimGain;
 
-    // If the gain is the best possible, no need to keep looking.
-    if (bestGain >= 0.0)
-      break;
+      // If the gain is the best possible, no need to keep looking.
+      if (bestGain >= 0.0)
+        break;
+    }
   }
 
   // Did we split or not?  If so, then split the data and create the children.
@@ -858,7 +969,7 @@ double DecisionTree<FitnessFunction,
       {
         child->Train<UseWeights>(data, currentChildBegin,
             currentCol - currentChildBegin, labels, numClasses, weights,
-            currentCol - currentChildBegin, minimumGainSplit,
+            currentCol - currentChildBegin, minimumGainSplit, maximumDepth - 1,
             dimensionSelector);
       }
       else
@@ -866,7 +977,8 @@ double DecisionTree<FitnessFunction,
         // During recursion entropy of child node may change.
         double childGain = child->Train<UseWeights>(data, currentChildBegin,
             currentCol - currentChildBegin, labels, numClasses, weights,
-            minimumLeafSize, minimumGainSplit, dimensionSelector);
+            minimumLeafSize, minimumGainSplit, maximumDepth - 1,
+            dimensionSelector);
         bestGain += double(childCounts[i]) / double(count) * (-childGain);
       }
       children.push_back(child);
@@ -883,6 +995,7 @@ double DecisionTree<FitnessFunction,
         numClasses,
         UseWeights ? weights.subvec(begin, begin + count - 1) : weights);
   }
+
   return -bestGain;
 }
 
