@@ -1,5 +1,5 @@
 /**
- * @file kde_impl.hpp
+ * @file methods/kde/kde_impl.hpp
  * @author Roberto Hueso
  *
  * Implementation of Kernel Density Estimation.
@@ -336,7 +336,16 @@ Evaluate(MatType querySet, arma::vec& estimations)
     std::vector<size_t> oldFromNewQueries;
     Tree* queryTree = BuildTree<Tree>(std::move(querySet), oldFromNewQueries);
     Timer::Stop("building_query_tree");
-    this->Evaluate(queryTree, oldFromNewQueries, estimations);
+    try
+    {
+      this->Evaluate(queryTree, oldFromNewQueries, estimations);
+    }
+    catch (std::exception& e)
+    {
+      // Make sure we delete the query tree.
+      delete queryTree;
+      throw;
+    }
     delete queryTree;
   }
   else if (mode == SINGLE_TREE_MODE)
