@@ -1,5 +1,5 @@
 /**
- * @file acrobot.hpp
+ * @file methods/reinforcement_learning/environment/acrobot.hpp
  * @author Rohan Raj
  *
  * This file is an implementation of Acrobot task:
@@ -85,19 +85,27 @@ class Acrobot
   /*
    * Implementation of action for Acrobot
    */
-  enum Action
+  class Action
   {
-    negativeTorque,
-    zeroTorque,
-    positiveTorque,
+   public:
+    enum actions
+    {
+      negativeTorque,
+      zeroTorque,
+      positiveTorque,
+    };
+    // To store the action.
+    Action::actions action;
 
     // Track the size of the action space.
-    size
+    static const size_t size = 3;
   };
 
   /**
    * Construct a Acrobot instance using the given constants.
    *
+   * @param maxSteps The number of steps after which the episode
+   *    terminates. If the value is 0, there is no limit.
    * @param gravity The gravity parameter.
    * @param linkLength1 The length of link 1.
    * @param linkLength2 The length of link 2.
@@ -110,10 +118,9 @@ class Acrobot
    * @param maxVel2 The max angular velocity of link2.
    * @param dt The differential value.
    * @param doneReward The reward recieved by the agent on success.
-   * @param maxSteps The number of steps after which the episode
-   *    terminates. If the value is 0, there is no limit.
    */
-  Acrobot(const double gravity = 9.81,
+  Acrobot(const size_t maxSteps = 500,
+          const double gravity = 9.81,
           const double linkLength1 = 1.0,
           const double linkLength2 = 1.0,
           const double linkMass1 = 1.0,
@@ -124,8 +131,8 @@ class Acrobot
           const double maxVel1 = 4 * M_PI,
           const double maxVel2 = 9 * M_PI,
           const double dt = 0.2,
-          const double doneReward = 0,
-          const size_t maxSteps = 0) :
+          const double doneReward = 0) :
+      maxSteps(maxSteps),
       gravity(gravity),
       linkLength1(linkLength1),
       linkLength2(linkLength2),
@@ -138,7 +145,6 @@ class Acrobot
       maxVel2(maxVel2),
       dt(dt),
       doneReward(doneReward),
-      maxSteps(maxSteps),
       stepsPerformed(0)
   { /* Nothing to do here */ }
 
@@ -169,10 +175,10 @@ class Acrobot
     nextState.Theta2() = Wrap(currentNextState[1], -M_PI, M_PI);
 
     //! The value of angular velocity is bounded in min and max value.
-    nextState.AngularVelocity1() = std::min(
-        std::max(currentNextState[2], -maxVel1), maxVel1);
-    nextState.AngularVelocity2() = std::min(
-        std::max(currentNextState[3], -maxVel2), maxVel2);
+    nextState.AngularVelocity1() = math::ClampRange(currentNextState[2],
+        -maxVel1, maxVel1);
+    nextState.AngularVelocity2() = math::ClampRange(currentNextState[3],
+        -maxVel2, maxVel2);
 
     // Check if the episode has terminated.
     bool done = IsTerminal(nextState);
@@ -193,7 +199,7 @@ class Acrobot
    *
    * @param state The current State.
    * @param action The action taken.
-   * @param nextState The next state.
+   * @return nextState The next state.
    */
   double Sample(const State& state, const Action& action)
   {
@@ -311,12 +317,12 @@ class Acrobot
    * This function calculates the torque for a particular action.
    * 0 : negative torque, 1 : zero torque, 2 : positive torque.
    *
-   * @param Action action taken.
+   * @param action Action taken.
    */
   double Torque(const Action& action) const
   {
     // Add noise to the Torque Torque is action number - 1. {0,1,2} -> {-1,0,1}.
-    return double(action - 1) + mlpack::math::Random(-0.1, 0.1);
+    return double(action.action - 1) + mlpack::math::Random(-0.1, 0.1);
   }
 
   /**
@@ -346,6 +352,9 @@ class Acrobot
   size_t& MaxSteps() { return maxSteps; }
 
  private:
+  //! Locally-stored maximum number of steps.
+  size_t maxSteps;
+
   //! Locally-stored gravity.
   double gravity;
 
@@ -382,9 +391,6 @@ class Acrobot
   //! Locally-stored done reward.
   double doneReward;
 
-  //! Locally-stored maximum number of steps.
-  size_t maxSteps;
-
   //! Locally-stored number of steps performed.
   size_t stepsPerformed;
 };
