@@ -1,5 +1,5 @@
 /**
- * @file q_learning_impl.hpp
+ * @file methods/reinforcement_learning/q_learning_impl.hpp
  * @author Shangtong Zhang
  *
  * This file is the implementation of QLearning class.
@@ -30,27 +30,30 @@ QLearning<
   UpdaterType,
   PolicyType,
   ReplayType
->::QLearning(TrainingConfig config,
-             NetworkType network,
-             PolicyType policy,
-             ReplayType replayMethod,
+>::QLearning(TrainingConfig& config,
+             NetworkType& network,
+             PolicyType& policy,
+             ReplayType& replayMethod,
              UpdaterType updater,
              EnvironmentType environment):
-    config(std::move(config)),
-    learningNetwork(std::move(network)),
+    config(config),
+    learningNetwork(network),
+    policy(policy),
+    replayMethod(replayMethod),
     updater(std::move(updater)),
     #if ENS_VERSION_MAJOR >= 2
     updatePolicy(NULL),
     #endif
-    policy(std::move(policy)),
-    replayMethod(std::move(replayMethod)),
     environment(std::move(environment)),
     totalSteps(0),
     deterministic(false)
 {
+  // To copy over the network structure.
+  targetNetwork = learningNetwork;
+
   // Set up q-learning network.
-  if (learningNetwork.Parameters().is_empty())
-    learningNetwork.ResetParameters();
+  learningNetwork.ResetParameters();
+  targetNetwork.ResetParameters();
 
   #if ENS_VERSION_MAJOR == 1
   this->updater.Initialize(learningNetwork.Parameters().n_rows,
@@ -62,7 +65,8 @@ QLearning<
                                    learningNetwork.Parameters().n_cols);
   #endif
 
-  targetNetwork = learningNetwork;
+  // Initialize the target network with the parameters of learning network.
+  targetNetwork.Parameters() = learningNetwork.Parameters();
 }
 
 template <
@@ -118,43 +122,22 @@ template <
   typename BehaviorPolicyType,
   typename ReplayType
 >
-double QLearning<
+void QLearning<
   EnvironmentType,
   NetworkType,
   UpdaterType,
   BehaviorPolicyType,
   ReplayType
->::Step()
+>::TrainAgent()
 {
-  // Get the action value for each action at current state.
-  arma::colvec actionValue;
-  learningNetwork.Predict(state.Encode(), actionValue);
-
-  // Select an action according to the behavior policy.
-  ActionType action = policy.Sample(actionValue, deterministic);
-
-  // Interact with the environment to advance to next state.
-  StateType nextState;
-  double reward = environment.Sample(state, action, nextState);
-
-  // Store the transition for replay.
-  replayMethod.Store(state, action, reward,
-      nextState, environment.IsTerminal(nextState));
-
-  // Update current state.
-  state = nextState;
-
-  if (deterministic || totalSteps < config.ExplorationSteps())
-    return reward;
-
   // Start experience replay.
 
   // Sample from previous experience.
   arma::mat sampledStates;
-  arma::icolvec sampledActions;
-  arma::colvec sampledRewards;
+  std::vector<ActionType> sampledActions;
+  arma::rowvec sampledRewards;
   arma::mat sampledNextStates;
-  arma::icolvec isTerminal;
+  arma::irowvec isTerminal;
 
   replayMethod.Sample(sampledStates, sampledActions, sampledRewards,
       sampledNextStates, isTerminal);
@@ -179,6 +162,9 @@ double QLearning<
   // Compute the update target.
   arma::mat target;
   learningNetwork.Forward(sampledStates, target);
+
+  double discount = std::pow(config.Discount(), replayMethod.NSteps());
+
   /**
    * If the agent is at a terminal state, then we don't need to add the
    * discounted reward. At terminal state, the agent wont perform any
@@ -186,20 +172,13 @@ double QLearning<
    */
   for (size_t i = 0; i < sampledNextStates.n_cols; ++i)
   {
-    if (isTerminal[i])
-    {
-      target(sampledActions(i), i) = sampledRewards(i);
-    }
-    else
-    {
-      target(sampledActions(i), i) = sampledRewards(i) + config.Discount() *
-          nextActionValues(bestActions(i), i);
-    }
+    target(sampledActions[i].action, i) = sampledRewards(i) + discount *
+        nextActionValues(bestActions(i), i) * (1 - isTerminal[i]);
   }
 
   // Learn from experience.
   arma::mat gradients;
-  learningNetwork.Backward(target, gradients);
+  learningNetwork.Backward(sampledStates, target, gradients);
 
   replayMethod.Update(target, sampledActions, nextActionValues, gradients);
 
@@ -210,7 +189,151 @@ double QLearning<
       gradients);
   #endif
 
-  return reward;
+  if (config.NoisyQLearning() == true)
+  {
+    learningNetwork.ResetNoise();
+    targetNetwork.ResetNoise();
+  }
+  // Update target network.
+  if (totalSteps % config.TargetNetworkSyncInterval() == 0)
+    targetNetwork.Parameters() = learningNetwork.Parameters();
+
+  if (totalSteps > config.ExplorationSteps())
+    policy.Anneal();
+}
+
+template <
+  typename EnvironmentType,
+  typename NetworkType,
+  typename UpdaterType,
+  typename BehaviorPolicyType,
+  typename ReplayType
+>
+void QLearning<
+  EnvironmentType,
+  NetworkType,
+  UpdaterType,
+  BehaviorPolicyType,
+  ReplayType
+>::TrainCategoricalAgent()
+{
+  // Start experience replay.
+
+  // Sample from previous experience.
+  arma::mat sampledStates;
+  std::vector<ActionType> sampledActions;
+  arma::rowvec sampledRewards;
+  arma::mat sampledNextStates;
+  arma::irowvec isTerminal;
+
+  replayMethod.Sample(sampledStates, sampledActions, sampledRewards,
+      sampledNextStates, isTerminal);
+
+  size_t atomSize = config.AtomSize();
+  arma::colvec support = arma::linspace<arma::colvec>(config.VMin(),
+      config.VMax(), atomSize);
+
+  size_t batchSize = sampledNextStates.n_cols;
+
+  // Compute action value for next state with target network.
+  arma::mat nextActionValues;
+  targetNetwork.Predict(sampledNextStates, nextActionValues);
+
+  arma::Col<size_t> nextAction;
+  if (config.DoubleQLearning())
+  {
+    // If use double Q-Learning, use learning network to select the best action.
+    arma::mat nextActionValues;
+    learningNetwork.Predict(sampledNextStates, nextActionValues);
+    nextAction = BestAction(nextActionValues);
+  }
+  else
+  {
+    nextAction = BestAction(nextActionValues);
+  }
+
+  arma::mat nextDists, nextDist(atomSize, batchSize);
+  targetNetwork.Forward(sampledNextStates, nextDists);
+  for (size_t i = 0; i < batchSize; ++i)
+  {
+    nextDist.col(i) = nextDists(nextAction(i) * atomSize, i,
+        arma::size(atomSize, 1));
+  }
+
+  arma::mat tZ = (arma::conv_to<arma::mat>::from(config.Discount() *
+      (support * (1 - isTerminal))).each_row() + sampledRewards);
+  tZ = arma::clamp(tZ, config.VMin(), config.VMax());
+  arma::mat b = (tZ - config.VMin()) / (config.VMax() - config.VMin()) *
+      (atomSize - 1);
+  arma::mat l = arma::floor(b);
+  arma::mat u = arma::ceil(b);
+
+  arma::mat projDistUpper = nextDist % (u - b);
+  arma::mat projDistLower = nextDist % (b - l);
+
+  arma::mat projDist = arma::zeros<arma::mat>(arma::size(nextDist));
+  for (size_t batchNo = 0; batchNo < batchSize; batchNo++)
+  {
+    for (size_t j = 0; j < atomSize; j++)
+    {
+      projDist(l(j, batchNo), batchNo) += projDistUpper(j, batchNo);
+      projDist(u(j, batchNo), batchNo) += projDistLower(j, batchNo);
+    }
+  }
+  arma::mat dists;
+  learningNetwork.Forward(sampledStates, dists);
+  arma::mat lossGradients = arma::zeros<arma::mat>(arma::size(dists));
+  for (size_t i = 0; i < batchSize; ++i)
+  {
+    lossGradients(sampledActions[i].action * atomSize, i,
+        arma::size(atomSize, 1)) = -(projDist.col(i) / (1e-10 + dists(
+        sampledActions[i].action * atomSize, i, arma::size(atomSize, 1))));
+  }
+  // Learn from experience.
+  arma::mat gradients;
+  learningNetwork.Backward(sampledStates, lossGradients, gradients);
+
+  #if ENS_VERSION_MAJOR == 1
+  updater.Update(learningNetwork.Parameters(), config.StepSize(), gradients);
+  #else
+  updatePolicy->Update(learningNetwork.Parameters(), config.StepSize(),
+      gradients);
+  #endif
+
+  if (config.NoisyQLearning() == true)
+  {
+    learningNetwork.ResetNoise();
+    targetNetwork.ResetNoise();
+  }
+  // Update target network.
+  if (totalSteps % config.TargetNetworkSyncInterval() == 0)
+    targetNetwork.Parameters() = learningNetwork.Parameters();
+
+  if (totalSteps > config.ExplorationSteps())
+    policy.Anneal();
+}
+
+template <
+  typename EnvironmentType,
+  typename NetworkType,
+  typename UpdaterType,
+  typename BehaviorPolicyType,
+  typename ReplayType
+>
+void QLearning<
+  EnvironmentType,
+  NetworkType,
+  UpdaterType,
+  BehaviorPolicyType,
+  ReplayType
+>::SelectAction()
+{
+  // Get the action value for each action at current state.
+  arma::colvec actionValue;
+  learningNetwork.Predict(state.Encode(), actionValue);
+
+  // Select an action according to the behavior policy.
+  action = policy.Sample(actionValue, deterministic, config.NoisyQLearning());
 }
 
 template <
@@ -231,34 +354,34 @@ double QLearning<
   // Get the initial state from environment.
   state = environment.InitialSample();
 
-  // Track the steps in this episode.
-  size_t steps = 0;
-
   // Track the return of this episode.
   double totalReturn = 0.0;
 
   // Running until get to the terminal state.
   while (!environment.IsTerminal(state))
   {
-    if (config.StepLimit() && steps >= config.StepLimit())
-      break;
+    SelectAction();
 
-    totalReturn += Step();
-    steps++;
-
-    if (deterministic)
-      continue;
+    // Interact with the environment to advance to next state.
+    StateType nextState;
+    double reward = environment.Sample(state, action, nextState);
 
+    totalReturn += reward;
     totalSteps++;
 
-    // Update target network
-    if (totalSteps % config.TargetNetworkSyncInterval() == 0)
-      targetNetwork = learningNetwork;
+    // Store the transition for replay.
+    replayMethod.Store(state, action, reward, nextState,
+        environment.IsTerminal(nextState), config.Discount());
+    // Update current state.
+    state = nextState;
 
-    if (totalSteps > config.ExplorationSteps())
-      policy.Anneal();
+    if (deterministic || totalSteps < config.ExplorationSteps())
+      continue;
+    if (config.IsCategorical())
+      TrainCategoricalAgent();
+    else
+      TrainAgent();
   }
-
   return totalReturn;
 }
 
@@ -266,4 +389,3 @@ double QLearning<
 } // namespace mlpack
 
 #endif
-
